@@ -12,33 +12,43 @@ This is the main widget used in the Plots plugin
 
 # Standard library imports
 import datetime
+import math
 import os.path as osp
 import sys
 
 # Third library imports
 from qtconsole.svg import svg_to_clipboard, svg_to_image
+from qtpy import PYQT5, PYQT6
 from qtpy.compat import getexistingdirectory, getsavefilename
-from qtpy.QtCore import QEvent, QPoint, QRect, QSize, Qt, QTimer, Signal, Slot
-from qtpy.QtGui import QPainter, QPixmap
-from qtpy.QtWidgets import (QApplication, QFrame, QGridLayout, QHBoxLayout,
+from qtpy.QtCore import (
+    QEvent,
+    QMimeData,
+    QPoint,
+    QRect,
+    QSize,
+    Qt,
+    QTimer,
+    Signal,
+    Slot,
+)
+from qtpy.QtGui import QDrag, QPainter, QPixmap
+from qtpy.QtWidgets import (QApplication, QFrame, QGridLayout, QLayout,
                             QScrollArea, QScrollBar, QSplitter, QStyle,
-                            QVBoxLayout, QWidget)
+                            QVBoxLayout, QWidget, QStackedLayout)
 
 # Local library imports
-from spyder.api.translations import get_translation
+from spyder.api.translations import _
 from spyder.api.widgets.mixins import SpyderWidgetMixin
 from spyder.utils.misc import getcwd_or_home
-from spyder.utils.palette import QStylePalette
+from spyder.utils.palette import SpyderPalette
+from spyder.utils.stylesheet import AppStyle
+from spyder.widgets.helperwidgets import PaneEmptyWidget
 
 
 # TODO:
 # - [ ] Generalize style updates, handle dark_interface with widget option
 
 
-# Localization
-_ = get_translation('spyder')
-
-
 def save_figure_tofile(fig, fmt, fname):
     """Save fig to fname in the format specified by fmt."""
     root, ext = osp.splitext(fname)
@@ -137,14 +147,19 @@ class FigureBrowser(QWidget, SpyderWidgetMixin):
     """
 
     def __init__(self, parent=None, background_color=None):
-        super().__init__(parent=parent, class_parent=parent)
+        if PYQT5 or PYQT6:
+            super().__init__(parent=parent, class_parent=parent)
+        else:
+            QWidget.__init__(self, parent)
+            SpyderWidgetMixin.__init__(self, class_parent=parent)
+
         self.shellwidget = None
-        self.is_visible = True
         self.figviewer = None
         self.setup_in_progress = False
         self.background_color = background_color
         self.mute_inline_plotting = None
         self.zoom_disp_value = None
+        self._update_when_shown = True
 
         # Setup the figure viewer.
         self.figviewer = FigureViewer(parent=self,
@@ -168,18 +183,33 @@ class FigureBrowser(QWidget, SpyderWidgetMixin):
         self.thumbnails_sb.sig_redirect_stdio_requested.connect(
             self.sig_redirect_stdio_requested)
 
+        # Widget empty pane
+        self.pane_empty = PaneEmptyWidget(
+            self,
+            "plots",
+            _("No plots to show"),
+            _("Run plot-generating code in the Editor or IPython console to "
+              "see your figures appear here. This pane only supports "
+              "static images, so it can't display interactive plots "
+              "like Bokeh, Plotly or Altair.")
+        )
+
         # Create the layout.
         self.splitter = splitter = QSplitter(parent=self)
         splitter.addWidget(self.figviewer)
         splitter.addWidget(self.thumbnails_sb)
         splitter.setFrameStyle(QScrollArea().frameStyle())
         splitter.setContentsMargins(0, 0, 0, 0)
+        splitter.setStyleSheet(
+            f"border-radius: {SpyderPalette.SIZE_BORDER_RADIUS}"
+        )
 
-        layout = QHBoxLayout(self)
-        layout.addWidget(splitter)
-        self.setLayout(layout)
-        layout.setContentsMargins(0, 0, 0, 0)
-        layout.setSpacing(0)
+        self.stack_layout = QStackedLayout()
+        self.stack_layout.addWidget(splitter)
+        self.stack_layout.addWidget(self.pane_empty)
+        self.setLayout(self.stack_layout)
+        self.stack_layout.setContentsMargins(0, 0, 0, 0)
+        self.stack_layout.setSpacing(0)
         self.setContentsMargins(0, 0, 0, 0)
 
     def _update_zoom_value(self, value):
@@ -192,15 +222,21 @@ class FigureBrowser(QWidget, SpyderWidgetMixin):
         """Setup the figure browser with provided options."""
         self.splitter.setContentsMargins(0, 0, 0, 0)
         for option, value in options.items():
-            if option == 'auto_fit_plotting':
-                self.change_auto_fit_plotting(value)
-            elif option == 'mute_inline_plotting':
+            if option == 'mute_inline_plotting':
                 self.mute_inline_plotting = value
+                if self.shellwidget:
+                    self.shellwidget.set_mute_inline_plotting(value)
             elif option == 'show_plot_outline':
                 self.show_fig_outline_in_viewer(value)
             elif option == 'save_dir':
                 self.thumbnails_sb.save_dir = value
 
+    def set_pane_empty(self, empty):
+        if empty:
+            self.stack_layout.setCurrentWidget(self.pane_empty)
+        else:
+            self.stack_layout.setCurrentWidget(self.splitter)
+
     def update_splitter_widths(self, base_width):
         """
         Update the widths to provide the scrollbar with a fixed minimum width.
@@ -219,29 +255,32 @@ class FigureBrowser(QWidget, SpyderWidgetMixin):
         if state is True:
             self.figviewer.figcanvas.setStyleSheet(
                 "FigureCanvas{border: 2px solid %s;}" %
-                QStylePalette.COLOR_BACKGROUND_4
+                SpyderPalette.COLOR_BACKGROUND_4
             )
         else:
             self.figviewer.figcanvas.setStyleSheet(
                 "FigureCanvas{border: 0px;}")
 
-    def change_auto_fit_plotting(self, state):
-        """Change the auto_fit_plotting option and scale images."""
-        self.figviewer.auto_fit_plotting = state
-
     def set_shellwidget(self, shellwidget):
         """Bind the shellwidget instance to the figure browser"""
         self.shellwidget = shellwidget
-        shellwidget.set_figurebrowser(self)
-        shellwidget.sig_new_inline_figure.connect(self._handle_new_figure)
+        self.shellwidget.set_mute_inline_plotting(self.mute_inline_plotting)
+        shellwidget.sig_new_inline_figure.connect(self.add_figure)
+        shellwidget.executing.connect(self._handle_new_execution)
 
-    def _handle_new_figure(self, fig, fmt):
+    def add_figure(self, fig, fmt):
         """
-        Handle when a new figure is sent to the IPython console by the
-        kernel.
+        Add a figure to the figure browser.
+
+        This is called when a new figure is sent to the IPython console by the
+        kernel, but can also be called directly.
         """
         self.thumbnails_sb.add_thumbnail(fig, fmt)
 
+    def _handle_new_execution(self):
+        """Handle a new execution in the console."""
+        self.thumbnails_sb._first_thumbnail_shown = False
+
     # ---- Toolbar Handlers
     def zoom_in(self):
         """Zoom the figure in by a single step in the figure viewer."""
@@ -286,6 +325,17 @@ class FigureBrowser(QWidget, SpyderWidgetMixin):
         if self.figviewer and self.figviewer.figcanvas.fig:
             self.figviewer.figcanvas.copy_figure()
 
+    # ---- Qt methods
+    def showEvent(self, event):
+        """Adjustments when the widget is shown."""
+        if self._update_when_shown:
+            # We only do this the first time the widget is shown to not change
+            # the splitter widths that users can set themselves.
+            self.update_splitter_widths(self.width())
+            self._update_when_shown = False
+
+        super().showEvent(event)
+
 
 class FigureViewer(QScrollArea, SpyderWidgetMixin):
     """
@@ -317,7 +367,12 @@ class FigureViewer(QScrollArea, SpyderWidgetMixin):
     """This signal is emitted when a new figure is loaded."""
 
     def __init__(self, parent=None, background_color=None):
-        super().__init__(parent, class_parent=parent)
+        if PYQT5 or PYQT6:
+            super().__init__(parent, class_parent=parent)
+        else:
+            QScrollArea.__init__(self, parent)
+            SpyderWidgetMixin.__init__(self, class_parent=parent)
+
         self.setAlignment(Qt.AlignCenter)
         self.viewport().setObjectName("figviewport")
         self.viewport().setStyleSheet(
@@ -325,7 +380,9 @@ class FigureViewer(QScrollArea, SpyderWidgetMixin):
         self.setFrameStyle(0)
 
         self.background_color = background_color
-        self._scalefactor = 0
+        self.current_thumbnail = None
+        self.scalefactor = 0
+
         self._scalestep = 1.2
         self._sfmax = 10
         self._sfmin = -10
@@ -336,6 +393,14 @@ class FigureViewer(QScrollArea, SpyderWidgetMixin):
         # An internal flag that tracks when the figure is being panned.
         self._ispanning = False
 
+        # To save scrollbar values in the current thumbnail
+        self.verticalScrollBar().valueChanged.connect(
+            self._set_vscrollbar_value
+        )
+        self.horizontalScrollBar().valueChanged.connect(
+            self._set_hscrollbar_value
+        )
+
     @property
     def auto_fit_plotting(self):
         """
@@ -349,13 +414,29 @@ class FigureViewer(QScrollArea, SpyderWidgetMixin):
         Set whether to automatically fit the plot to the scroll area size.
         """
         self._auto_fit_plotting = value
+
+        if self.current_thumbnail is not None:
+            self.current_thumbnail.auto_fit = value
+
         if value:
+            self.scale_image()
             self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
             self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
         else:
             self.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)
             self.setHorizontalScrollBarPolicy(Qt.ScrollBarAsNeeded)
-        self.scale_image()
+
+    @property
+    def scalefactor(self):
+        """Return the current scale factor."""
+        return self._scalefactor
+
+    @scalefactor.setter
+    def scalefactor(self, value):
+        """Set the scale factor value."""
+        self._scalefactor = value
+        if self.current_thumbnail is not None:
+            self.current_thumbnail.scalefactor = value
 
     def setup_figcanvas(self):
         """Setup the FigureCanvas."""
@@ -373,13 +454,44 @@ class FigureViewer(QScrollArea, SpyderWidgetMixin):
             point = self.figcanvas.mapToGlobal(qpoint)
             self.sig_context_menu_requested.emit(point)
 
+    def set_current_thumbnail(self, thumbnail):
+        """Set the current thumbnail displayed in the viewer."""
+        self.current_thumbnail = thumbnail
+
     def load_figure(self, fig, fmt):
         """Set a new figure in the figure canvas."""
+        self.auto_fit_plotting = self.current_thumbnail.auto_fit
+
+        # Let scale_image compute the scale factor for the thumbnail if it
+        # hasn't one yet.
+        if self.current_thumbnail.scalefactor is not None:
+            self.scalefactor = self.current_thumbnail.scalefactor
+
         self.figcanvas.load_figure(fig, fmt)
         self.sig_figure_loaded.emit()
         self.scale_image()
         self.figcanvas.repaint()
 
+        # Save the computed scale factor by scale_image in the thumbnail
+        if self.current_thumbnail.scalefactor is None:
+            self.current_thumbnail.scalefactor = self.scalefactor
+
+        # Restore scrollbar values.
+        # We need to use timers for this because trying to set those values
+        # immediately after the figure is loaded doesn't work.
+        QTimer.singleShot(
+            20,
+            self.update_scrollbars_values,
+        )
+
+    def update_scrollbars_values(self):
+        self.verticalScrollBar().setValue(
+            self.current_thumbnail.vscrollbar_value
+        )
+        self.horizontalScrollBar().setValue(
+            self.current_thumbnail.hscrollbar_value
+        )
+
     def eventFilter(self, widget, event):
         """A filter to control the zooming and panning of the figure canvas."""
 
@@ -402,15 +514,22 @@ class FigureViewer(QScrollArea, SpyderWidgetMixin):
         # ---- Panning
         # Set ClosedHandCursor:
         elif event.type() == QEvent.MouseButtonPress:
-            if event.button() == Qt.LeftButton:
-                QApplication.setOverrideCursor(Qt.ClosedHandCursor)
+            if (
+                event.button() == Qt.LeftButton
+                and not self.auto_fit_plotting
+                and (
+                    self.verticalScrollBar().isVisible()
+                    or self.horizontalScrollBar().isVisible()
+                )
+            ):
+                self.setCursor(Qt.ClosedHandCursor)
                 self._ispanning = True
                 self.xclick = event.globalX()
                 self.yclick = event.globalY()
 
         # Reset Cursor:
         elif event.type() == QEvent.MouseButtonRelease:
-            QApplication.restoreOverrideCursor()
+            self.setCursor(Qt.ArrowCursor)
             self._ispanning = False
 
         # Move  ScrollBar:
@@ -428,22 +547,37 @@ class FigureViewer(QScrollArea, SpyderWidgetMixin):
                 scrollBarV = self.verticalScrollBar()
                 scrollBarV.setValue(scrollBarV.value() + dy)
 
+        # Show in full size
+        elif (
+            event.type() == QEvent.MouseButtonDblClick
+            and self.scalefactor != 0
+        ):
+            self.auto_fit_plotting = False
+            self.zoom_in(to_full_size=True)
+
+            # Necessary to correctly set the state of the fit_action button
+            self.sig_figure_loaded.emit()
+
         return QWidget.eventFilter(self, widget, event)
 
     # ---- Figure Scaling Handlers
-    def zoom_in(self):
+    def zoom_in(self, to_full_size=False):
         """Scale the image up by one scale step."""
-        if self._scalefactor <= self._sfmax:
-            self._scalefactor += 1
+        # This is necessary so that the scale factor becomes zero below
+        if to_full_size:
+            self.scalefactor = -1
+
+        if self.scalefactor <= self._sfmax:
+            self.scalefactor += 1
             self.scale_image()
             self._adjust_scrollbar(self._scalestep)
 
     def zoom_out(self):
         """Scale the image down by one scale step."""
-        if self._scalefactor >= self._sfmin:
-            self._scalefactor -= 1
+        if self.scalefactor >= self._sfmin:
+            self.scalefactor -= 1
             self.scale_image()
-            self._adjust_scrollbar(1/self._scalestep)
+            self._adjust_scrollbar(1 / self._scalestep)
 
     def scale_image(self):
         """Scale the image size."""
@@ -452,8 +586,8 @@ class FigureViewer(QScrollArea, SpyderWidgetMixin):
 
         # Don't auto fit plotting
         if not self.auto_fit_plotting:
-            new_width = int(fwidth * self._scalestep ** self._scalefactor)
-            new_height = int(fheight * self._scalestep ** self._scalefactor)
+            new_width = int(fwidth * self._scalestep ** self.scalefactor)
+            new_height = int(fheight * self._scalestep ** self.scalefactor)
 
         # Auto fit plotting
         # Scale the image to fit the figviewer size while respecting the ratio.
@@ -466,7 +600,9 @@ class FigureViewer(QScrollArea, SpyderWidgetMixin):
             height = (size.height() -
                       style.pixelMetric(QStyle.PM_LayoutTopMargin) -
                       style.pixelMetric(QStyle.PM_LayoutBottomMargin))
+
             self.figcanvas.setToolTip('')
+
             try:
                 if (fwidth / fheight) > (width / height):
                     new_width = int(width)
@@ -478,21 +614,39 @@ class FigureViewer(QScrollArea, SpyderWidgetMixin):
                 icon = self.create_icon('broken_image')
                 self.figcanvas._qpix_orig = icon.pixmap(fwidth, fheight)
                 self.figcanvas.setToolTip(
-                    _('The image is broken, please try to generate it again'))
+                    _('The image is broken, please try to generate it again')
+                )
                 new_width = fwidth
                 new_height = fheight
+                self.auto_fit_plotting = False
 
         if self.figcanvas.size() != QSize(new_width, new_height):
             self.figcanvas.setFixedSize(new_width, new_height)
+
+            # Adjust the scale factor according to the scaling of the fitted
+            # image. This is necessary so that zoom in/out increases/decreases
+            # the image size in factors of of +1/-1 of the one computed below.
+            if self.auto_fit_plotting:
+                self.scalefactor = self.get_scale_factor()
+
             self.sig_zoom_changed.emit(self.get_scaling())
 
     def get_scaling(self):
         """Get the current scaling of the figure in percent."""
-        return round(self.figcanvas.width() / self.figcanvas.fwidth * 100)
+        width = self.figcanvas.width()
+        fwidth = self.figcanvas.fwidth
+        if fwidth != 0:
+            return round(width / fwidth * 100)
+        else:
+            return 100
+
+    def get_scale_factor(self):
+        """Get scale factor according to the current scaling."""
+        return math.log(self.get_scaling() / 100) / math.log(self._scalestep)
 
     def reset_original_image(self):
         """Reset the image to its original size."""
-        self._scalefactor = 0
+        self.scalefactor = 0
         self.scale_image()
 
     def _adjust_scrollbar(self, f):
@@ -508,6 +662,16 @@ class FigureViewer(QScrollArea, SpyderWidgetMixin):
         vb = self.verticalScrollBar()
         vb.setValue(int(f * vb.value() + ((f - 1) * vb.pageStep()/2)))
 
+    def _set_vscrollbar_value(self, value):
+        """Save vertical scrollbar value in current thumbnail."""
+        if self.current_thumbnail is not None:
+            self.current_thumbnail.vscrollbar_value = value
+
+    def _set_hscrollbar_value(self, value):
+        """Save horizontal scrollbar value in current thumbnail."""
+        if self.current_thumbnail is not None:
+            self.current_thumbnail.hscrollbar_value = value
+
 
 class ThumbnailScrollBar(QFrame):
     """
@@ -515,7 +679,7 @@ class ThumbnailScrollBar(QFrame):
     created when a figure is sent to the IPython console by the kernel and
     that controls what is displayed in the FigureViewer.
     """
-    _min_scrollbar_width = 100
+    _min_scrollbar_width = 130
 
     # Signals
     sig_redirect_stdio_requested = Signal(bool)
@@ -568,10 +732,14 @@ class ThumbnailScrollBar(QFrame):
         # is added to the scrollarea, we need to do it instead in a slot
         # connected to the scrollbar's rangeChanged signal.
         # See spyder-ide/spyder#10914 for more details.
-        self._new_thumbnail_added = False
+        self._scroll_to_last_thumbnail = False
+        self._first_thumbnail_shown = False
         self.scrollarea.verticalScrollBar().rangeChanged.connect(
             self._scroll_to_newest_item)
 
+        # To reorganize thumbnails with drag and drop
+        self.setAcceptDrops(True)
+
     def setup_gui(self):
         """Setup the main layout of the widget."""
         layout = QVBoxLayout(self)
@@ -581,20 +749,26 @@ class ThumbnailScrollBar(QFrame):
 
     def setup_scrollarea(self):
         """Setup the scrollarea that will contain the FigureThumbnails."""
-        self.view = QWidget()
+        self.view = QWidget(self)
+
+        self.scene = QVBoxLayout(self.view)
+        self.scene.setAlignment(Qt.AlignHCenter | Qt.AlignTop)
+        self.scene.setContentsMargins(
+            0, AppStyle.MarginSize, 0, AppStyle.MarginSize
+        )
 
-        self.scene = QGridLayout(self.view)
-        self.scene.setContentsMargins(0, 0, 0, 0)
         # The vertical spacing between the thumbnails.
         # Note that we need to set this value explicitly or else the tests
         # are failing on macOS. See spyder-ide/spyder#11576.
-        self.scene.setSpacing(5)
+        self.scene.setSpacing(2 * AppStyle.MarginSize)
 
-        self.scrollarea = QScrollArea()
+        self.scrollarea = QScrollArea(self)
         self.scrollarea.setWidget(self.view)
         self.scrollarea.setWidgetResizable(True)
         self.scrollarea.setFrameStyle(0)
-        self.scrollarea.setViewportMargins(2, 2, 2, 2)
+        self.scrollarea.setViewportMargins(
+            AppStyle.MarginSize, 0, AppStyle.MarginSize, 0
+        )
         self.scrollarea.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
         self.scrollarea.setMinimumWidth(self._min_scrollbar_width)
 
@@ -633,6 +807,54 @@ class ThumbnailScrollBar(QFrame):
             self._update_thumbnail_size()
         return super().eventFilter(widget, event)
 
+    def dragEnterEvent(self, event):
+        """Enable drag events on this widget."""
+        event.accept()
+
+    def dropEvent(self, event):
+        """
+        Handle drop events.
+
+        Solution adapted from
+        https://www.pythonguis.com/faq/pyqt-drag-drop-widgets
+        """
+        # Event variables
+        pos = event.pos()
+        dropped_thumbnail = event.source()
+
+        # Avoid accepting drops from other widgets
+        if not isinstance(dropped_thumbnail, FigureThumbnail):
+            return
+
+        # Main variables
+        scrollbar_pos = self.scrollarea.verticalScrollBar().value()
+        n_thumbnails = self.scene.count()
+        last_thumbnail = self.scene.itemAt(n_thumbnails - 1).widget()
+
+        # Move thumbnail
+        if (pos.y() + scrollbar_pos) > last_thumbnail.y():
+            # This allows to move a thumbnail to the last position
+            self.scene.insertWidget(n_thumbnails - 1, dropped_thumbnail)
+        else:
+            # This works for any other position, including the first one
+            for i in range(n_thumbnails):
+                w = self.scene.itemAt(i).widget()
+
+                if (
+                    (pos.y() + scrollbar_pos)
+                    < (w.y() + w.size().height() // 5)
+                ):
+                    self.scene.insertWidget(i - 1, dropped_thumbnail)
+                    break
+
+        # Recreate thumbnails list to take into account the new order
+        # Fixes spyder-ide/spyder#22458
+        self._thumbnails = []
+        for i in range(n_thumbnails):
+            self._thumbnails.append(self.scene.itemAt(i).widget())
+
+        event.accept()
+
     # ---- Save Figure
     def save_all_figures_as(self):
         """Save all the figures to a file."""
@@ -749,8 +971,18 @@ class ThumbnailScrollBar(QFrame):
         """
         Add a new thumbnail to that thumbnail scrollbar.
         """
+        # Always stick at end for the first thumbnail
+        is_first = not self._first_thumbnail_shown
+
+        # Stick at end if we are already at the end
+        stick_at_end = False
+        sb = self.scrollarea.verticalScrollBar()
+        if sb.value() == sb.maximum():
+            stick_at_end = True
+
         thumbnail = FigureThumbnail(
-            parent=self, background_color=self.background_color)
+            parent=self, background_color=self.background_color
+        )
         thumbnail.canvas.load_figure(fig, fmt)
         thumbnail.sig_canvas_clicked.connect(self.set_current_thumbnail)
         thumbnail.sig_remove_figure_requested.connect(self.remove_thumbnail)
@@ -758,16 +990,26 @@ class ThumbnailScrollBar(QFrame):
         thumbnail.sig_context_menu_requested.connect(
             lambda point: self.show_context_menu(point, thumbnail))
         self._thumbnails.append(thumbnail)
-        self._new_thumbnail_added = True
+        self.scene.addWidget(thumbnail)
 
-        self.scene.setRowStretch(self.scene.rowCount() - 1, 0)
-        self.scene.addWidget(thumbnail, self.scene.rowCount() - 1, 0)
-        self.scene.setRowStretch(self.scene.rowCount(), 100)
-        self.set_current_thumbnail(thumbnail)
+        self._scroll_to_last_thumbnail = True
+        self._first_thumbnail_shown = True
+
+        # Only select a new thumbnail if the last one was selected
+        select_last = (
+            len(self._thumbnails) < 2
+            or self.current_thumbnail == self._thumbnails[-2]
+            or is_first
+        )
+        if select_last:
+            self.set_current_thumbnail(thumbnail)
 
         thumbnail.show()
         self._setup_thumbnail_size(thumbnail)
 
+        if not is_first and not stick_at_end:
+            self._scroll_to_last_thumbnail = False
+
     def remove_current_thumbnail(self):
         """Remove the currently selected thumbnail."""
         if self.current_thumbnail is not None:
@@ -786,6 +1028,7 @@ class ThumbnailScrollBar(QFrame):
 
         self._thumbnails = []
         self.current_thumbnail = None
+        self.figure_viewer.auto_fit_plotting = False
         self.figure_viewer.figcanvas.clear_canvas()
 
     def remove_thumbnail(self, thumbnail):
@@ -804,7 +1047,7 @@ class ThumbnailScrollBar(QFrame):
         if thumbnail in self._thumbnails:
             self._thumbnails.remove(thumbnail)
 
-        # Select a new thumbnail if any :
+        # Select a new thumbnail, if any
         if thumbnail == self.current_thumbnail:
             if len(self._thumbnails) > 0:
                 self.set_current_index(
@@ -843,11 +1086,16 @@ class ThumbnailScrollBar(QFrame):
 
     def set_current_thumbnail(self, thumbnail):
         """Set the currently selected thumbnail."""
+        if self.current_thumbnail == thumbnail:
+            return
+        if self.current_thumbnail is not None:
+            self.current_thumbnail.highlight_canvas(False)
         self.current_thumbnail = thumbnail
+        self.figure_viewer.set_current_thumbnail(thumbnail)
         self.figure_viewer.load_figure(
-            thumbnail.canvas.fig, thumbnail.canvas.fmt)
-        for thumbnail in self._thumbnails:
-            thumbnail.highlight_canvas(thumbnail == self.current_thumbnail)
+            thumbnail.canvas.fig, thumbnail.canvas.fmt
+        )
+        self.current_thumbnail.highlight_canvas(True)
 
     def go_previous_thumbnail(self):
         """Select the thumbnail previous to the currently selected one."""
@@ -867,7 +1115,7 @@ class ThumbnailScrollBar(QFrame):
 
     def scroll_to_item(self, index):
         """Scroll to the selected item of ThumbnailScrollBar."""
-        spacing_between_items = self.scene.verticalSpacing()
+        spacing_between_items = self.scene.spacing()
         height_view = self.scrollarea.viewport().height()
         height_item = self.scene.itemAt(index).sizeHint().height()
         height_view_excluding_item = max(0, height_view - height_item)
@@ -890,8 +1138,8 @@ class ThumbnailScrollBar(QFrame):
         Note that this method is called each time the rangeChanged signal
         is emitted by the scrollbar.
         """
-        if self._new_thumbnail_added:
-            self._new_thumbnail_added = False
+        if self._scroll_to_last_thumbnail:
+            self._scroll_to_last_thumbnail = False
             self.scrollarea.verticalScrollBar().setValue(vsb_max)
 
     # ---- ScrollBar Handlers
@@ -956,10 +1204,18 @@ class FigureThumbnail(QWidget):
         The QPoint in global coordinates where the menu was requested.
     """
 
-    def __init__(self, parent=None, background_color=None):
+    def __init__(self, parent=None, background_color=None, auto_fit=True):
         super().__init__(parent)
-        self.canvas = FigureCanvas(parent=self,
-                                   background_color=background_color)
+
+        self.auto_fit = auto_fit
+        self.scalefactor = None
+        self.vscrollbar_value = 0
+        self.hscrollbar_value = 0
+
+        self.canvas = FigureCanvas(
+            parent=self,
+            background_color=background_color
+        )
         self.canvas.sig_context_menu_requested.connect(
             self.sig_context_menu_requested)
         self.canvas.installEventFilter(self)
@@ -970,18 +1226,17 @@ class FigureThumbnail(QWidget):
         layout = QGridLayout(self)
         layout.setContentsMargins(0, 0, 0, 0)
         layout.addWidget(self.canvas, 0, 0, Qt.AlignCenter)
-        layout.setSizeConstraint(layout.SetFixedSize)
+        layout.setSizeConstraint(QLayout.SizeConstraint.SetFixedSize)
 
     def highlight_canvas(self, highlight):
         """
         Set a colored frame around the FigureCanvas if highlight is True.
         """
         if highlight:
-            # Highlighted figure is not clear in dark mode with blue color.
-            # See spyder-ide/spyder#10255.
+            # See spyder-ide/spyder#21598 for choice of styling.
             self.canvas.setStyleSheet(
-                "FigureCanvas{border: 2px solid %s;}" %
-                QStylePalette.COLOR_ACCENT_3
+                "FigureCanvas{border: 3px solid %s;}" %
+                SpyderPalette.COLOR_ACCENT_3
             )
         else:
             self.canvas.setStyleSheet("FigureCanvas{}")
@@ -993,13 +1248,14 @@ class FigureThumbnail(QWidget):
         """
         fwidth = self.canvas.fwidth
         fheight = self.canvas.fheight
-        if fwidth / fheight > 1:
-            canvas_width = max_canvas_size
-            canvas_height = canvas_width / fwidth * fheight
-        else:
-            canvas_height = max_canvas_size
-            canvas_width = canvas_height / fheight * fwidth
-        self.canvas.setFixedSize(int(canvas_width), int(canvas_height))
+        if fheight != 0:
+            if fwidth / fheight > 1:
+                canvas_width = max_canvas_size
+                canvas_height = canvas_width / fwidth * fheight
+            else:
+                canvas_height = max_canvas_size
+                canvas_width = canvas_height / fheight * fwidth
+            self.canvas.setFixedSize(int(canvas_width), int(canvas_height))
         self.layout().setColumnMinimumWidth(0, max_canvas_size)
 
     def eventFilter(self, widget, event):
@@ -1013,6 +1269,27 @@ class FigureThumbnail(QWidget):
 
         return super().eventFilter(widget, event)
 
+    def mouseMoveEvent(self, event):
+        """
+        Enable drags to reorganize thumbnails with the mouse in the scrollbar.
+
+        Solution taken from:
+        https://www.pythonguis.com/faq/pyqt-drag-drop-widgets/
+        """
+        if event.buttons() == Qt.LeftButton:
+            # Create drag
+            drag = QDrag(self)
+            mime = QMimeData()
+            drag.setMimeData(mime)
+
+            # Show pixmap of the thumbnail while it's being moved.
+            pixmap = QPixmap(self.size())
+            self.render(pixmap)
+            drag.setPixmap(pixmap)
+
+            # Execute drag's event loop
+            drag.exec_(Qt.MoveAction)
+
 
 class FigureCanvas(QFrame):
     """
